; #########################################################################

;    +=============================+=====++
;    |     Written by Ernie Murphy | '00 ||
;    \     Tweaked by Limo Driver  | '07 ||
;     ^============================+=====++

; #########################################################################

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap : none ; case sensitive

; #########################################################################

    include     \masm32\include\ole32.inc
    includelib  \masm32\lib\ole32.lib

; #########################################################################

    IMalloc         PROTO
    Alloc           PROTO :DWORD
    Realloc         PROTO :DWORD, :DWORD
    GetSize         PROTO :DWORD
    Free            PROTO :DWORD
    DidAlloc        PROTO :DWORD
    HeapMinimize    PROTO

; #########################################################################
   
    IMallocProc MACRO procName, procOffset, paramCount
    ;{
        if paramCount eq 0
        ;{
            procName PROC public
        ;}
        elseif paramCount eq 1
        ;{
            procName PROC public p1 : DWORD
        ;}
        elseif paramCount eq 2
        ;{
            procName PROC public p1 : DWORD, p2 : DWORD
        ;}
        endif
        ;{
            invoke  IMalloc
           
            .IF eax
            ;{
                if paramCount gt 1
                ;{
                    push    DWORD PTR [ p2 ]
                ;}
                endif

                if paramCount gt 0
                ;{
                    push    DWORD PTR [ p1 ]
                ;}
                endif
           
                push    eax
                mov     eax, [ eax ]
                call    DWORD PTR [ eax + procOffset ]
            ;}
            .ENDIF
       
            ret
        ;}
        procName ENDP
    ;}
    ENDM
       
; #########################################################################

    .data

    Alloc_pIMalloc dd 0

    .code

; #########################################################################

    ;--------------------------------------------------------------------------
    ; IMalloc returns the interface pointer to the memory allocator.
    ; This interface pointer cannot be used from a remote process -
    ; each process must have its own allocator.
    ;
    ; If an error occured while fetching the interface pointer,
    ; IMalloc returns NULL.
    ;
    ; EXAMPLE:
    ; invoke IMalloc ; returns interface pointer in eax
    ;
    ; Uses: eax, ecx, edx
    ;--------------------------------------------------------------------------

    IMalloc proc public
    ;{
        lea     ecx, [ Alloc_pIMalloc ]
        mov     eax, [ ecx ]

        .IF eax
        ;{
            ret
        ;}
        .ENDIF

    @@: invoke  CoGetMalloc, 1, ecx
        or      eax, eax
        jz      IMalloc
        ;{
            xor eax, eax
            ret
        ;}
    ;}
    IMalloc endp

; #########################################################################

    ;--------------------------------------------------------------------------
    ; The Alloc function uses the default allocator to allocate a memory block
    ; in the same way that malloc does. The initial contents of the returned
    ; memory block are undefined  there is no guarantee that the block has
    ; been initialized. The allocated block may be larger than cb bytes because
    ; of the space required for alignment and for maintenance information.
    ;
    ; If cb is zero, Alloc allocates a zero-length item and returns a valid
    ; pointer to that item. If there is insufficient memory available, Alloc
    ; returns NULL.
    ;
    ; Applications should always check the return value from this method, even
    ; when requesting small amounts of memory, because there is no guarantee
    ; the memory will be allocated.
    ;
    ; EXAMPLE:
    ; invoke Alloc, 128 ; allocates 128 bytes, returns memory pointer in eax
    ;
    ; Uses: eax, ecx, edx
    ;--------------------------------------------------------------------------

    IMallocProc Alloc,        12, 1

; #########################################################################

    ;--------------------------------------------------------------------------
    ; The Realloc function changes the size of a memory block allocated by
    ; a previous call to Alloc or Realloc, in the same way that realloc does.
    ;
    ; The pv argument points to the beginning of the memory block.
    ; If pv is NULL, Realloc allocates a new memory block in the same way as
    ; the Alloc function. If pv is not NULL, it should be a pointer returned by
    ; a prior call to Alloc or Realloc.
    ;
    ; The cb argument specifies the size (in bytes) of the new block. The
    ; contents of the block are unchanged up to the shorter of the new and old
    ; sizes, although the new block can be in a different location. Because the
    ; new block can be in a different memory location, the pointer returned by
    ; Realloc is not guaranteed to be the pointer passed through the pv argument.
    ;
    ; If pv is not NULL and cb is zero, then the memory pointed to by pv is
    ; freed, with a return value of NULL.
    ;
    ; If pv is not NULL, cb is a valid size, and there was not enough memory
    ; available to expand the block to the given size, Realloc will return NULL,
    ; and the original block will remain unchanged.
    ;
    ; EXAMPLE:
    ; invoke Realloc, pv, 256 ; reallocates the memory at pv, pointer in eax
    ;
    ; Uses: eax, ecx, edx
    ;--------------------------------------------------------------------------

    IMallocProc Realloc,      16, 2

; #########################################################################

    ;--------------------------------------------------------------------------
    ; The Free function, using the default OLE allocator, frees a block of memory
    ; previously allocated through a call to the Alloc or Realloc function.
    ;
    ; The number of bytes freed equals the number of bytes that were originally
    ; allocated or reallocated. After the call, the memory block pointed to by
    ; pv is invalid and can no longer be used.
    ;
    ; The pv parameter can be NULL. If so, this method has no effect.
    ;
    ; EXAMPLE:
    ; invoke Free, pv ; frees the memory at pv, doesn't return a value (void)
    ;
    ; Uses: eax, ecx, edx
    ;--------------------------------------------------------------------------

    IMallocProc Free,         20, 1

; #########################################################################

    ;--------------------------------------------------------------------------
    ; The GetSize function returns the size of a memory block allocated by a
    ; previous call to Alloc or Realloc. If pv is a NULL pointer, the return
    ; value is -1.
    ;
    ; The size returned is the actual size of the allocation, which may be
    ; greater than the size requested when the allocation was made due to
    ; space required for alignment and for maintenance information.
    ;
    ; EXAMPLE:
    ; invoke GetSize, pv ; returns the memory block size (in bytes) in eax
    ;
    ; Uses: eax, ecx, edx
    ;--------------------------------------------------------------------------

    IMallocProc GetSize,      24, 1

; #########################################################################

    ;--------------------------------------------------------------------------
    ; The DidAlloc function is useful if a application is using multiple allocations and
    ; needs to know whether a previously allocated block of memory was allocated by a
    ; particular allocation.
    ;
    ; Return values can be 1, 0, or -1, signifying the following:
    ;
    ;  1 - The memory block was allocated by this IMalloc instance;
    ;  0 - The memory block was not allocated by this IMalloc instance;
    ; -1 - DidAlloc is unable to determine whether or not it allocated the memory block.
    ;
    ; If DidAlloc has been called with a NULL pointer, it will return -1.
    ;
    ; EXAMPLE:
    ; invoke DidAlloc, pv ; pv is a memory pointer, return value in eax
    ;
    ; Uses: eax, ecx, edx
    ;--------------------------------------------------------------------------

    IMallocProc DidAlloc,     28, 1

; #########################################################################

    ;--------------------------------------------------------------------------
    ; The HeapMinimize function minimizes the heap as much as possible by
    ; releasing unused memory to the operating system, coalescing adjacent
    ; free blocks and committing free pages.
    ;
    ; Calling HeapMinimize is useful when an application has been running for
    ; some time and the heap may be fragmented.
    ;
    ; EXAMPLE:
    ; invoke HeapMinimize ; releases unused memory to the operating system
    ;
    ; Uses: eax, ecx, edx
    ;--------------------------------------------------------------------------

    IMallocProc HeapMinimize, 32, 0

; #########################################################################

end